package sun.security.provider;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.security.NoSuchProviderException;
import java.security.NoSuchAlgorithmException;
import java.security.MessageDigest;
import java.io.Serializable;
import java.security.SecureRandomSpi;

public final class SecureRandom extends SecureRandomSpi implements Serializable
{
    private static final long serialVersionUID = 3581829991155417889L;
    private static final int DIGEST_SIZE = 20;
    private transient MessageDigest digest;
    private byte[] state;
    private byte[] remainder;
    private int remCount;
    
    public SecureRandom() {
        this.init((byte[])null);
    }
    
    private SecureRandom(byte[] array) {
        this.init(array);
    }
    
    private void init(byte[] array) {
        try {
            this.digest = MessageDigest.getInstance("SHA", "SUN");
        }
        catch (NoSuchProviderException | NoSuchAlgorithmException ex) {
            try {
                this.digest = MessageDigest.getInstance("SHA");
            }
            catch (NoSuchAlgorithmException cause) {
                throw new InternalError("internal error: SHA-1 not available.", cause);
            }
        }
        if (array != null) {
            this.engineSetSeed(array);
        }
    }
    
    public byte[] engineGenerateSeed(int n) {
        byte[] array = new byte[n];
        SeedGenerator.generateSeed(array);
        return array;
    }
    
    public synchronized void engineSetSeed(byte[] input) {
        if (this.state != null) {
            this.digest.update(this.state);
            for (int i = 0; i < this.state.length; ++i) {
                this.state[i] = 0;
            }
        }
        this.state = this.digest.digest(input);
    }
    
    private static void updateState(byte[] array, byte[] array2) {
        int n = 1;
        boolean n2 = false;
        for (int i = 0; i < array.length; ++i) {
            int n3 = array[i] + array2[i] + n;
            int n4 = (byte)n3;
            n2 |= (array[i] != n4);
            array[i] = n4;
            n = n3 >> 8;
        }
        if (n2 == 0) {
            int n5 = 0;
            ++array[n5];
        }
    }
    
    public synchronized void engineNextBytes(byte[] array) {
        int i = 0;
        byte[] remainder = this.remainder;
        if (this.state == null) {
            byte[] input = new byte[20];
            SecureRandom.SeederHolder.seeder.engineNextBytes(input);
            this.state = this.digest.digest(input);
        }
        int remCount = this.remCount;
        if (remCount > 0) {
            int n = (array.length - i < 20 - remCount) ? (array.length - i) : (20 - remCount);
            for (int j = 0; j < n; ++j) {
                array[j] = remainder[remCount];
                remainder[remCount++] = 0;
            }
            this.remCount += n;
            i += n;
        }
        while (i < array.length) {
            this.digest.update(this.state);
            remainder = this.digest.digest();
            updateState(this.state, remainder);
            int n2 = (array.length - i > 20) ? 20 : (array.length - i);
            for (int k = 0; k < n2; ++k) {
                array[i++] = remainder[k];
                remainder[k] = 0;
            }
            this.remCount += n2;
        }
        this.remainder = remainder;
        this.remCount %= 20;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        try {
            this.digest = MessageDigest.getInstance("SHA", "SUN");
        }
        catch (NoSuchProviderException | NoSuchAlgorithmException ex) {
            try {
                this.digest = MessageDigest.getInstance("SHA");
            }
            catch (NoSuchAlgorithmException cause) {
                throw new InternalError("internal error: SHA-1 not available.", cause);
            }
        }
    }
    
    SecureRandom(byte[] array, SecureRandom.1 object) {
        this(array);
    }
    
    private static class SeederHolder
    {
        private static final SecureRandom seeder;
        
        static {
            seeder = new SecureRandom(SeedGenerator.getSystemEntropy());
            byte[] array = new byte[20];
            SeedGenerator.generateSeed(array);
            SecureRandom.SeederHolder.seeder.engineSetSeed(array);
        }
    }
}
